 ##############################################################################
#
# Latest eggdrop.fr Script In Topic
# v2.02 (18/04/2020)   ©2013-2020 MenzAgitat
#
# IRC: irc.epiknet.org  #boulets / #eggdrop
#
# Mes scripts sont téléchargeables sur http://www.eggdrop.fr
# Retrouvez aussi toute l'actualité de mes releases sur
# http://www.boulets.oqp.me/tcl/scripts/index.html
#
 ##############################################################################

#
# Description
#
# Le but de ce script est de récupérer à intervalles réguliers les informations
# concernant le dernier téléchargement disponible sur www.eggdrop.fr, et de les
# injecter dans le topic de votre chan via Topic Warden (script du même auteur à
# télécharger séparément).
# La 1ère analyse est déclenchée 3mn après le chargement du script, et ce afin
# de ne pas surcharger l'eggdrop de tâches dès son lancement.
#
# Voici un exemple de masque de topic que vous pouvez utiliser dans Topic Warden
# pour utiliser ce script :
# ...::::::::: Entraide pour les eggdrops, tcl et shells -| http://www.eggdrop.fr |- :::::::::...  -| téléchargement le plus récent : %latestscriptname% - %latestscripturl%
#
 ###############################################################################

#
# Changelog
#
# 1.0
#		- 1ère version
# 1.1
#		- Correction : le script ne fonctionnait plus suite à une mise à jour de la
#			section téléchargements du site www.eggdrop.fr
#		- Passage sous licence Creative Commons
# 1.11
#		- Modification de la procédure de raccourcissement des URLs par TinyURL pour
#			utiliser l'API.
#	2.0
#		- Correction : si un script était publié puis retiré, le topic ne se mettait
#			pas à jour pour refléter la modification.
#		- Modification : la connexion au site web s'effectue désormais de manière
#			asynchrone.
#		- Ajout : nouveaux paramètres de configuration EggdropFr_timeout,
#			TinyURL_timeout et id_file.
#		- Diverses optimisations et améliorations du code, notamment mise au propre
#			de la gestion des exceptions HTTP.
# 2.01
#		- Correction : le script refusait de se charger sur les Eggdrops v1.8+
#		- Correction : en cas d'indisponiblité du site eggdrop.fr, le script
#			rencontrait une erreur.
#		- Modification : le timer a été abandonné au profit d'un crontab, référez-
#			vous au paramètre crontab dans la section configuration.
# 2.02
#		- Correction d'une bourde de ma part dans la précédente release.
#
 ###############################################################################

#
# Licence
#
#		Cette création est mise à disposition selon le Contrat
#		Attribution-NonCommercial-ShareAlike 3.0 Unported disponible en ligne
#		http://creativecommons.org/licenses/by-nc-sa/3.0/ ou par courrier postal à
#		Creative Commons, 171 Second Street, Suite 300, San Francisco, California
#		94105, USA.
#		Vous pouvez également consulter la version française ici :
#		http://creativecommons.org/licenses/by-nc-sa/3.0/deed.fr
#
 ###############################################################################

if {[::tcl::info::commands ::EFRSAnnouncer::unload] eq "::EFRSAnnouncer::unload"} { ::EFRSAnnouncer::unload }
if { [package vcompare [lindex [split $::version] 0] 1.6.20] == -1 } { putloglev o * "\00304\[Latest Eggdrop.fr Script in Topic - Erreur\]\003 La version de votre Eggdrop est\00304 ${::version}\003; Latest Eggdrop.fr Script in Topic ne fonctionnera correctement que sur les Eggdrops version 1.6.20 ou supérieure." ; return }
if { [::tcl::info::tclversion] < 8.5 } { putloglev o * "\00304\[Latest Eggdrop.fr Script in Topic - Erreur\]\003 Latest Eggdrop.fr Script in Topic nécessite que Tcl 8.5 (ou plus) soit installé pour fonctionner. Votre version actuelle de Tcl est\00304 ${::tcl_version}\003." ; return }
package require Tcl 8.5
package require http
namespace eval ::EFRSAnnouncer {


 ###############################################################################
### Configuration
 ###############################################################################

	# Nom du chan sur lequel on doit annoncer les news (le script est conçu pour
	# ne fonctionner que sur un chan)
	variable announcer_chan "#eggdrop"

	# Intervalle de temps entre deux lectures du site web (en minutes)
	# Remarque : Il s'agit d'une chaîne de crontab. Vous trouverez des détails
	# concernant la syntaxe ainsi que de nombreux exemples ici :
	#	http://fr.wikipedia.org/wiki/Crontab
	# Avec le réglage par défaut, l'information est rafraîchie toutes les 15mn.
	variable crontab "*/15 * * * *"
	
	# Après combien de temps (en secondes) abandonne-t-on la tentative de
	# connexion si le site web n'a pas répondu assez rapidement ?
	variable EggdropFr_timeout 6

	# Après combien de secondes décide-t-on que le site web www.tinyurl.com est
	# offline (ou trop lent) en l'absence de réponse de sa part ?
	# Remarque : en cas de timeout, l'url intégrale sera affichée à la place
	# de sa version raccourcie.
	variable TinyURL_timeout 5

	# Nom des variables à renseigner dans le masque du topic :
	#  variable contenant le nom du script
	variable TW_scriptname_variable "%latestscriptname%"
	#  variable contenant l'url de téléchargement
	variable TW_scripturl_variable "%latestscripturl%"

	# Nom et emplacement du fichier qui contiendra l'id du dernier téléchargement
	variable id_file "scripts/latest_eggdropfr_script_in_topic.id"



 ###############################################################################
### Fin de la configuration
 ###############################################################################



	 #############################################################################
	### Initialisation
	 #############################################################################
	variable scriptname "Latest Eggdrop.fr Script in Topic"
	variable version "2.02.20200418"
	variable useragent "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/20100101 Firefox/23.0"
	variable downloads_url "http://rss.eggdrop.fr/downloads.xml"
	proc unload {args} {
		putlog "Désallocation des ressources de $::EFRSAnnouncer::scriptname..."
		foreach binding [lsearch -inline -all -regexp [binds *[set ns [::tcl::string::range [namespace current] 2 end]]*] " \{?(::)?$ns"] {
			unbind [lindex $binding 0] [lindex $binding 1] [lindex $binding 2] [lindex $binding 4]
		}
		foreach running_timer [timers] {
			if { [::tcl::string::match "*[namespace current]::*" [lindex $running_timer 1]] } { killtimer [lindex $running_timer 2] }
		}
		namespace delete ::EFRSAnnouncer
	}
}

 ###############################################################################
### Envoi d'une requête au site web
 ###############################################################################
proc ::EFRSAnnouncer::check_website {args} {
	::http::config -useragent $::EFRSAnnouncer::useragent
	if { [catch { set token [::http::geturl $::EFRSAnnouncer::downloads_url -command ::EFRSAnnouncer::process_received_data -timeout [expr {$::EFRSAnnouncer::EggdropFr_timeout * 1000}]] }] } {
		::EFRSAnnouncer::error_msg 1
	} else {
		return
	}
}

 ###############################################################################
### Le site web a répondu, traitement de la réponse
 ###############################################################################
proc ::EFRSAnnouncer::process_received_data {token} {
		switch -- [::http::status $token] {
		"error" {
			::EFRSAnnouncer::error_msg 1
			return
		}
		"eof" {
			::EFRSAnnouncer::error_msg 2 $token
			return
		}
		"ok" {
			# on parse le flux RSS à la recherche du dernier téléchargement disponible
			if { [set payload [regexp -inline {<\!\[CDATA\[([^\]]*?)\]\]></title>\s+<link>(.*?)</link>\s+<pubDate>(.*?)</pubDate>\s+<guid isPermaLink=(?:.*?)</guid>\s+<description><\!\[CDATA\[(.*?)\]\]></description>\s+<category>(.*)</category>\s+<author>(.*)</author>} [encoding convertfrom utf-8 [::http::data $token]]]] eq "" } {
				::EFRSAnnouncer::error_msg 3 $token
				return
			} else {
				if { [file exists $::EFRSAnnouncer::id_file] } {
					set file_ID [open $::EFRSAnnouncer::id_file r]
					gets $file_ID latest_download_date
					close $file_ID
				} else {
					set latest_download_date 0
				}
				lassign $payload {} script_name download_url date description category author
				set date [::tcl::clock::scan [::tcl::string::range $date 0 end-6] -format "%a, %d %b %Y %H:%M:%S"]
				if { $date == $latest_download_date } {
					::http::cleanup $token
					return
				} else {
					if { [set download_url [::EFRSAnnouncer::tinyurl_conversion "$download_url"]] == 0 } {
						::http::cleanup $token
						return
					} else {
						::topic_control {Latest Eggdrop.fr Script in Topic} $EFRSAnnouncer::announcer_chan set $EFRSAnnouncer::TW_scriptname_variable [::EFRSAnnouncer::html_filter $script_name] $EFRSAnnouncer::TW_scripturl_variable $download_url
						set file_ID [open $::EFRSAnnouncer::id_file w]
						puts $file_ID $date
						close $file_ID
					}
				}
			}
		}
	}
	::http::cleanup $token
	return
}

 ###############################################################################
### Affichage des messages d'erreur
 ###############################################################################
proc ::EFRSAnnouncer::error_msg {id token} {
	switch -- $id {
		1 {
			putloglev o * "\00304\[$::EFRSAnnouncer::scriptname - erreur\]\003 La connexion au site web n'a pas pu être établie."
		}
		2 {
			putloglev o * "\00304\[$::EFRSAnnouncer::scriptname - erreur\]\003 La connexion au site web a été interrompue par le serveur."
		}
		3 {
			putloglev o * "\00304\[$::EFRSAnnouncer::scriptname - erreur\]\003 Aucune donnée n'a été reçue dans le temps imparti."
		}
		4 {
			putloglev o * "\00304\[$::EFRSAnnouncer::scriptname - erreur\]\003 La connexion à www.tinyurl.com n'a pu être établie, la modification du topic a été reportée."
		}
	}
	::http::cleanup $token
}

 ###############################################################################
### Conversion des entités html spéciaux
 ###############################################################################
proc ::EFRSAnnouncer::html_filter { data } {
	return [::tcl::string::map {
		"&agrave;"		"à"		"&agrave;"		"à"		"&aacute;"		"á"		"&acirc;"			"â"
		"&atilde;"		"ã"		"&auml;"			"ä"		"&aring;"			"å"		"&aelig;"			"æ"
		"&ccedil;"		"ç"		"&egrave;"		"è"		"&eacute;"		"é"		"&ecirc;"			"ê"
		"&euml;"			"ë"		"&igrave;"		"ì"		"&iacute;"		"í"		"&icirc;"			"î"
		"&iuml;"			"ï"		"&eth;"				"ð"		"&ntilde;"		"ñ"		"&ograve;"		"ò"
		"&oacute;"		"ó"		"&ocirc;"			"ô"		"&otilde;"		"õ"		"&ouml;"			"ö"
		"&divide;"		"÷"		"&oslash;"		"ø"		"&ugrave;"		"ù"		"&uacute;"		"ú"
		"&ucirc;"			"û"		"&uuml;"			"ü"		"&yacute;"		"ý"		"&thorn;"			"þ"
		"&yuml;"			"ÿ"		"&quot;"			"\""	"&amp;"				"&"		"&euro;"			"€"
		"&oelig;"			"œ"		"&Yuml;"			"Ÿ"		"&iexcl;"			"¡"
		"&cent;"			"¢"		"&pound;"			"£"		"&curren;"		"¤"		"&yen;"				"¥"
		"&brvbar;"		"¦"		"&brkbar;"		"¦"		"&sect;"			"§"		"&uml;"				"¨"
		"&die;"				"¨"		"&copy;"			"©"		"&ordf;"			"ª"		"&laquo;"			"«"
		"&not;"				"¬"		"&shy;"				"­-"	"&reg;"				"®"		"&macr;"			"¯"
		"&hibar;"			"¯"		"&deg;"				"°"		"&plusmn;"		"±"		"&sup2;"			"²"
		"&sup3;"			"³"		"&acute;"			"´"		"&micro;"			"µ"		"&para;"			"¶"
		"&middot;"		"·"		"&cedil;"			"¸"		"&sup1;"			"¹"		"&ordm;"			"º"
		"&raquo;"			"»"		"&frac14;"		"¼"		"&frac12;"		"½"		"&frac34;"		"¾"
		"&iquest;"		"¿"		"&Agrave;"		"À"		"&Aacute;"		"Á"		"&Acirc;"			"Â"
		"&Atilde;"		"Ã"		"&Auml;"			"Ä"		"&Aring;"			"Å"		"&AElig;"			"Æ"
		"&Ccedil;"		"Ç"		"&Egrave;"		"È"		"&Eacute;"		"É"		"&Ecirc;"			"Ê"
		"&Euml;"			"Ë"		"&Igrave;"		"Ì"		"&Iacute;"		"Í"		"&Icirc;"			"Î"
		"&Iuml;"			"Ï"		"&ETH;"				"Ð"		"&Dstrok;"		"Ð"		"&Ntilde;"		"Ñ"
		"&Ograve;"		"Ò"		"&Oacute;"		"Ó"		"&Ocirc;"			"Ô"		"&Otilde;"		"Õ"
		"&Ouml;"			"Ö"		"&times;"			"×"		"&Oslash;"		"Ø"		"&Ugrave;"		"Ù"
		"&Uacute;"		"Ú"		"&Ucirc;"			"Û"		"&Uuml;"			"Ü"		"&Yacute;"		"Ý"
		"&THORN;"			"Þ"		"&szlig;"			"ß"		"\r"					"\n"	"\t"					""
		"&#039;"			"\'"	"&#39;"				"\'"	"&nbsp;"			" "		"&nbsp"				" "
		"&#34;"				"\'"	"&#38;"				"&"		"#91;"				"\("	"&#92;"				"\/"
		"&#93;"				")"		"&#123;"			"("		"&#125;"			")"		"&#163;"			"£"
		"&#168;"			"¨"		"&#169;"			"©"		"&#171;"			"«"		"&#173;"			"­"
		"&#174;"			"®"		"&#180;"			"´"		"&#183;"			"·"		"&#185;"			"¹"
		"&#187;"			"»"		"&#188;"			"¼"		"&#189;"			"½"		"&#190;"			"¾"
		"&#192;"			"À"		"&#193;"			"Á"		"&#194;"			"Â"		"&#195;"			"Ã"
		"&#196;"			"Ä"		"&#197;"			"Å"		"&#198;"			"Æ"		"&#199;"			"Ç"
		"&#200;"			"È"		"&#201;"			"É"		"&#202;"			"Ê"		"&#203;"			"Ë"
		"&#204;"			"Ì"		"&#205;"			"Í"		"&#206;"			"Î"		"&#207;"			"Ï"
		"&#208;"			"Ð"		"&#209;"			"Ñ"		"&#210;"			"Ò"		"&#211;"			"Ó"
		"&#212;"			"Ô"		"&#213;"			"Õ"		"&#214;"			"Ö"		"&#215;"			"×"
		"&#216;"			"Ø"		"&#217;"			"Ù"		"&#218;"			"Ú"		"&#219;"			"Û"
		"&#220;"			"Ü"		"&#221;"			"Ý"		"&#222;"			"Þ"		"&#223;"			"ß"
		"&#224;"			"à"		"&#225;"			"á"		"&#226;"			"â"		"&#227;"			"ã"
		"&#228;"			"ä"		"&#229;"			"å"		"&#230;"			"æ"		"&#231;"			"ç"
		"&#232;"			"è"		"&#233;"			"é"		"&#234;"			"ê"		"&#235;"			"ë"
		"&#236;"			"ì"		"&#237;"			"í"		"&#238;"			"î"		"&#239;"			"ï"
		"&#240;"			"ð"		"&#241;"			"ñ"		"&#242;"			"ò"		"&#243;"			"ó"
		"&#244;"			"ô"		"&#245;"			"õ"		"&#246;"			"ö"		"&#247;"			"÷"
		"&#248;"			"ø"		"&#249;"			"ù"		"&#250;"			"ú"		"&#251;"			"û"
		"&#252;"			"ü"		"&#253;"			"ý"		"&#254;"			"þ"		"&#9830;"			""
		"&lt;"				"<"		"&gt;"				">"
	} $data]
}

 ###############################################################################
### Conversion de l'URL en une version plus courte
 ###############################################################################
proc ::EFRSAnnouncer::tinyurl_conversion {url} {
  set url [::tcl::string::map -nocase {"&amp;" "&"} $url]
	set query "http://tinyurl.com/api-create.php?[::http::formatQuery url ${url}]"
	if { ![catch { set token [::http::geturl $query -timeout [expr $::EFRSAnnouncer::TinyURL_timeout * 1000]] }] } {
		set short_url [::http::data $token]
		if { ($short_url != 0) && ($short_url ne "") } {
			::http::cleanup $token
			return $short_url
		}
	}
	::EFRSAnnouncer::error_msg 4 $token
	return 0
}

 ###############################################################################
### Binds
 ###############################################################################
bind evnt - prerehash ::EFRSAnnouncer::unload
# On attend 3mn avant d'activer le bind cron afin d'éviter que l'Eggdrop soit
# surchargé de tâches dès son démarrage.
timer 3 { bind cron - $::EFRSAnnouncer::crontab ::EFRSAnnouncer::check_website }

putlog "$::EFRSAnnouncer::scriptname v$::EFRSAnnouncer::version (©2013-2020 MenzAgitat) a été chargé."